# -*- coding: utf-8 -*-
import json
import random
import re

import cv2

random.seed(42)


def convert(text):
    return int(text) if text.isdigit() else text.lower()


def natural_sort(data):
    return sorted(data, key=lambda key: [convert(c) for c in re.split("([0-9]+)", key)])


def assign_random_split(train_prob, val_prob):
    """
    assuming train_prob + val_prob + test_prob = 1
    """
    prob = random.random()
    if prob <= train_prob:
        return "train"
    elif prob <= train_prob + val_prob:
        return "val"
    else:
        return "test"


def save_text(path, text):
    with open(path, "w") as f:
        f.write(text)


def save_image(path, image):
    cv2.imwrite(path, cv2.cvtColor(image, cv2.COLOR_BGR2RGB))


def save_json(path, dict):
    with open(path, "w") as outfile:
        json.dump(dict, outfile, indent=4)
